@@ -69,24 +69,15 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   //m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
 
 
-  //m_Paned.set_border_width(6);
-  m_Paned.set_position(200); //Set size of top pane.
-  pack_start(m_Paned);
-  m_Paned.add(m_FlowTable);
-
-  //Related records:
-  /*
-  m_Label_Related.set_text(Bakery::App_Gtk::util_bold_message(_("Related Records")));
-  m_Label_Related.set_use_markup(true);
-  m_Frame_Related.set_label_widget(m_Label_Related);
-  m_Frame_Related.set_shadow_type(Gtk::SHADOW_NONE);  
-  m_Frame_Related.add(m_Alignment_Related);
-  m_Alignment_Related.set_padding(0, 0, 12, 0);
-  
-  m_Notebook_Related.set_border_width(6);
-  m_Alignment_Related.add(m_Notebook_Related);
-  m_Paned.add(m_Frame_Related);
-  */
+  //m_ScrolledWindow.set_border_width(6);
+  m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC); /* Allow vertical scrolling, but never scroll horizontally. */
+  m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_NONE); //SHADOW_IN is Recommended by the GNOME HIG, but looks odd.
+  pack_start(m_ScrolledWindow);
+  m_ScrolledWindow.add(m_FlowTable);
+  // The FlowTable does not support native scrolling, so gtkmm adds it to a
+  // viewport first that also has some shadow we do not want.
+  Gtk::Viewport* viewport = dynamic_cast<Gtk::Viewport*>(m_FlowTable.get_parent());
+  if(viewport) viewport->set_shadow_type(Gtk::SHADOW_NONE);
 
   //Add or delete record:
   m_HBox.pack_start(m_Button_New, Gtk::PACK_SHRINK);
@@ -196,6 +187,11 @@ bool Box_Data_Details::fill_from_database()
 {
   //std::cout << "Box_Data_Details::fill_from_database(): m_primary_key_value.to_string()=" << m_primary_key_value.to_string() << std::endl;
 
+ //Don't try to open a connection if there is no document,
+ //for instance, during application destruction.
+ if(!get_document())
+    return false;
+
   bool bResult = false;
 
   Bakery::BusyCursor busy_cursor(get_app_window());
@@ -548,13 +544,7 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
   if(layout_item)
   {
     const Gnome::Gda::Value primary_key_value = get_primary_key_value();
-    const type_map_fields field_values = get_record_field_values_for_calculation(m_table_name, m_field_primary_key, primary_key_value);
-
-   //We need the connection when we run the script, so that the script may use it.
-   sharedptr<SharedConnection> sharedconnection = connect_to_server(0 /* parent window */);
-
-    glom_execute_python_function_implementation(layout_item->get_script(), field_values, //TODO: Maybe use the field's type here.
-    get_document(), get_table_name(), sharedconnection->get_gda_connection());
+    execute_button_script(layout_item, primary_key_value);
 
     //Refresh the view, in case the script changed any data:
     if(get_primary_key_is_in_foundset(m_found_set, m_primary_key_value)) //Check, because maybe the script deleted the current record, or changed something so that it should no longer be shown in the found set.
@@ -568,7 +558,7 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
     }
   }
 }
-
+     
 void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
