@@ -19,10 +19,15 @@
  */
 
 #include "box_db_table_definition.h"
+#include <glom/frame_glom.h>
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <glom/glom_postgres.h>
 #include "../../../config.h"
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Box_DB_Table_Definition::Box_DB_Table_Definition()
 {
   init();
@@ -168,7 +173,7 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
       //This must match the SQL statement above:
       sharedptr<Field> field(new Field());
       field->set_name(name);
-      field->set_title( util_title_from_string(name) ); //Start with a title that might be useful.
+      field->set_title( Utils::title_from_string(name) ); //Start with a title that might be useful.
       field->set_glom_type(Field::TYPE_NUMERIC);
 
       Gnome::Gda::FieldAttributes field_info = field->get_field_info();
@@ -207,6 +212,67 @@ void Box_DB_Table_Definition::on_adddel_delete(const Gtk::TreeModel::iterator& r
   fill_from_database();
 }
 
+bool Box_DB_Table_Definition::check_field_change(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field_new)
+{
+  bool result = true; //Tells the caller whether to continue.
+
+  Gtk::Window* parent_window = get_app_window();
+
+  //Warn about a long slow recalculation, and offer the chance to cancel it:
+  if(field_new->get_has_calculation())
+  {
+    if(field_new->get_calculation() != field_old->get_calculation())
+    {
+      //TODO: Only show this when there are > 100 records?
+      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Recalculation Required")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+      dialog.set_secondary_text(_("You have changed the calculation used by this field so Glom must recalculate the value in all records. If the table contains many records then this could take a long time."));
+      if(parent_window)
+        dialog.set_transient_for(*parent_window);
+
+      dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+      dialog.add_button(_("Recalculate"), Gtk::RESPONSE_OK);
+      result = (dialog.run() == Gtk::RESPONSE_OK);
+    }
+  }
+
+  //If we are changing a non-glom type:
+  //Refuse to edit field definitions that were not created by glom:
+  if(Field::get_glom_type_for_gda_type( field_old->get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
+  {
+    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Invalid database structure")), true);
+    dialog.set_secondary_text(_("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
+    if(parent_window)
+       dialog.set_transient_for(*parent_window);
+
+    dialog.run();
+
+    result = false;
+  }
+  else
+  {
+    //Refuse to set a second primary key:
+    //bool bcontinue = true;
+    if(field_new->get_primary_key() && !field_old->get_primary_key()) //Was the primary key column checked?
+    {
+      //Is there an existing primary key?
+      sharedptr<Field> existing_primary_key = get_field_primary_key_for_table(m_table_name);
+      if(existing_primary_key)
+      {
+        //Warn the user and refuse to make the change:
+        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Too many primary keys")), true);
+        dialog.set_secondary_text(_("You may not specify more than one field as the primary key."));
+        if(parent_window)
+          dialog.set_transient_for(*parent_window);
+        dialog.run();
+
+        result = false;
+      }
+    }
+  }
+
+  return result;
+}
+
 void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator& row, guint /* col */)
 {
   //Get old field definition:
@@ -231,22 +297,19 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
       //Change it:
       if(*m_Field_BeingEdited != *fieldNew) //If it has really changed.
       {
-        //If we are changing a non-glom type:
-        //Refuse to edit field definitions that were not created by glom:
-        if(Field::get_glom_type_for_gda_type( m_Field_BeingEdited->get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
-        {
-          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Invalid database structure")), true);
-          dialog.set_secondary_text(_("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
-          //TODO: dialog.set_transient_for(*get_application());
-          dialog.run();
-        }
-        else
+        const bool bcontinue = check_field_change(m_Field_BeingEdited, fieldNew);
+        if(bcontinue)
         {
           sharedptr<Field> fieldNewWithModifications = change_definition(m_Field_BeingEdited, fieldNew);
 
           //Update the row to show any extra changes (such as unique being set/unset whenever the primary key is set/unset) 
           fill_field_row(row, fieldNewWithModifications);
         }
+        else
+        {
+          //revert:
+          fill_field_row(row, m_Field_BeingEdited);
+        }
       }
     }
   }
@@ -261,6 +324,10 @@ void Box_DB_Table_Definition::on_adddel_edit(const Gtk::TreeModel::iterator& row
 
   //m_pDialog->set_modified(false); //Disable [Apply] at start.
 
+  Gtk::Window* parent_window = get_app_window();
+  if(parent_window)
+    m_pDialog->set_transient_for(*parent_window);
+
   m_pDialog->show();
 }
 
@@ -337,38 +404,11 @@ void Box_DB_Table_Definition::on_Properties_apply()
 
   if(*m_Field_BeingEdited != *field_New)
   {
-    //If we are changing a non-glom type:
-    //Refuse to edit field definitions that were not created by glom:
-    if( Field::get_glom_type_for_gda_type(m_Field_BeingEdited->get_field_info().get_gdatype()) == Field::TYPE_INVALID )
+    const bool bcontinue = check_field_change(m_Field_BeingEdited, field_New);
+    if(bcontinue)
     {
-      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Invalid database structure")), true);
-         dialog.set_secondary_text(_("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
-      //TODO: dialog.set_transient_for(*get_app_window());
-      dialog.run();
-    }
-    else
-    {
-      //Warn about recalculation:
-      bool change_def = true;
-      if(field_New->get_has_calculation())
-      {
-        if(field_New->get_calculation() != m_Field_BeingEdited->get_calculation())
-        {
-          //TODO: Only show this when there are > 100 records?
-          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Recalculation Required")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
-          dialog.set_secondary_text(_("You have changed the calculation used by this field so Glom must recalculate the value in all records. If the table contains many records then this could take a long time."));
-          //dialog.set_transient_for(*get_app_window());
-          dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-          dialog.add_button(_("Recalculate"), Gtk::RESPONSE_OK);
-          change_def = (dialog.run() == Gtk::RESPONSE_OK);
-        }
-      }
-
-      if(change_def)
-      {
-        change_definition(m_Field_BeingEdited, field_New);
-        m_Field_BeingEdited = field_New;
-      }
+      change_definition(m_Field_BeingEdited, field_New);
+      m_Field_BeingEdited = field_New;
     }
 
     //Update the list:
@@ -382,9 +422,48 @@ sharedptr<Field> Box_DB_Table_Definition::change_definition(const sharedptr<cons
 {
   Bakery::BusyCursor busy_cursor(get_app_window());
 
-  //DB field defintion:
+  //DB field definition:
 
   sharedptr<Field> result;
+  
+  if(!fieldOld || !field)
+    return result;
+
+  if(fieldOld->get_primary_key() != field->get_primary_key())
+  {
+    //If setting the primarykeyness, check whether this is likely to succeed:
+    if(!fieldOld->get_primary_key() && field->get_primary_key())
+    {
+      //Check for nulls:
+      if(field_has_null_values(fieldOld)) //Use the fieldOld because we only use the name, and we want to check the _existing_ field:
+      {
+        Gtk::Window* window = const_cast<Gtk::Window*>(get_app_window());
+        if(window)
+          Frame_Glom::show_ok_dialog(_("Field contains empty values."), _("The field may not yet be used as a primary key because it contains empty values."), *window, Gtk::MESSAGE_WARNING);
+
+          //TODO: Offer to fill it in with generated ID numbers? That could give strange results if the existing values are human-readable.
+
+        return glom_sharedptr_clone(fieldOld); //No changes were made.
+      }
+
+      //Check that the values are unique:
+      if(field_has_non_unique_values(fieldOld)) //Use the fieldOld because we only use the name, and we want to check the _existing_ field:
+      {
+        Gtk::Window* window = const_cast<Gtk::Window*>(get_app_window());
+        if(window)
+          Frame_Glom::show_ok_dialog(_("Field contains non-unique values."), _("The field may not yet be used as a primary key because it contains values that are not unique."), *window, Gtk::MESSAGE_WARNING);
+
+        return glom_sharedptr_clone(fieldOld); //No changes were made.
+      }
+
+   
+    }
+
+    //Forget the remembered currently-viewed primary key value, 
+    //because it will be useless with a different field as the primary key, or with no field as primary key:
+    Document_Glom* document = get_document();
+    document->forget_layout_record_viewed(m_table_name);
+  }
 
   try
   {
@@ -478,14 +557,18 @@ sharedptr<Field> Box_DB_Table_Definition::postgres_change_column(const sharedptr
   else
   {
     //Change other stuff, without changing the type:
-    return postgres_change_column_extras(m_table_name, field_old, field);
+    return GlomPostgres::postgres_change_column_extras(m_table_name, field_old, field);
   }
 }
 
 
-void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
+void Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
 {
-  sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window());
+  Gtk::Window* parent_window = get_app_window();
+  if(!parent_window)
+    return;
+
+  sharedptr<SharedConnection> sharedconnection = connect_to_server(parent_window);
   if(sharedconnection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
@@ -512,7 +595,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
 
         sharedptr<Field> fieldTemp = glom_sharedptr_clone(field);
         fieldTemp->set_name("glom_temp_column");
-        postgres_add_column(m_table_name, fieldTemp); //This might also involves several commands.
+        GlomPostgres::postgres_add_column(m_table_name, fieldTemp); //This might also involves several commands.
 
 
         bool conversion_failed = false;
@@ -622,11 +705,72 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
       }   //TODO: Abandon the transaction if something failed.
     }  /// If the datatype has changed:
 
-    if(!new_column_created) //We don't need to change anything else if everything was alrady correctly set as a new column:
-      postgres_change_column_extras(m_table_name, field_old, field);
+    if(!new_column_created) //We don't need to change anything else if everything was already correctly set as a new column:
+      GlomPostgres::postgres_change_column_extras(m_table_name, field_old, field);
   }
 }
 
+bool Box_DB_Table_Definition::field_has_null_values(const sharedptr<const Field>& field)
+{
+  //Note that "= Null" doesn't work, though it doesn't error either.
+  //Note also that SELECT COUNT always returns 0 if all the values are NULL, so we can't use that to be more efficient.
+  const Glib::ustring sql_query = "SELECT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\" WHERE \"" + m_table_name + "\".\"" + field->get_name() + "\" IS NULL ";
+  //std::cout << "sql_query: " << sql_query << std::endl;
+
+  long null_count = 0;
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query, get_app_window());
+  if(datamodel && datamodel->get_n_rows() && datamodel->get_n_columns())
+  {
+    null_count = datamodel->get_n_rows();
+    //std::cout << "debug: null_count = " << null_count << std::endl;
+  }
+  else
+  {
+    g_warning("Box_DB_Table_Definition::field_has_null_values(): SELECT COUNT() failed.");
+  }
+
+   return null_count > 0; 
+}
+
+bool Box_DB_Table_Definition::field_has_non_unique_values(const sharedptr<const Field>& field)
+{
+  long count_distinct = 0;
+  long count_all = 0;
+
+  //Count the distint rows:
+  const Glib::ustring sql_query_distinct = "SELECT DISTINCT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\"";
+  
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query_distinct, get_app_window());
+  if(datamodel)
+  {
+    count_distinct = datamodel->get_n_rows();
+    //std::cout << "debug: null_count = " << null_count << std::endl;
+  }
+  else
+  {
+    g_warning("Box_DB_Table_Definition::field_has_non_unique_values(): SELECT COUNT() failed.");
+  }
+
+  //Count all rows, to compare. TODO_performance: Is there a more efficient way to do this? Maybe count(*), which apparently doesn't ignore NULL rows like count(somefield) would.
+  const Glib::ustring sql_query_all = "SELECT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\"";
+  
+  datamodel = query_execute(sql_query_all, get_app_window());
+  if(datamodel)
+  {
+    count_all = datamodel->get_n_rows();
+    //std::cout << "debug: null_count = " << null_count << std::endl;
+  }
+  else
+  {
+    g_warning("Box_DB_Table_Definition::field_has_non_unique_values(): SELECT COUNT() failed.");
+  }
+
+
+  return count_distinct != count_all;
+}
+
+
+} //namespace Glom
 
 
 
